@@ -24,12 +24,6 @@ import sys
 from gdb_socket import GDBSocket
 from gdb_websocket import GDBWebSocket
 
-"""
-import os
-parentdir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
-sys.path.insert(0,parentdir)
-"""
-
 SIGINT = (2)
 SIGSEGV = (11)
 SIGILL = (4)
@@ -73,7 +67,7 @@ class GDBServer(threading.Thread):
         else:
             self.port = port_urlWSS
         self.packet_size = 2048
-        self.flashData = ""
+        self.flashData = list()
         self.conn = None
         self.lock = threading.Lock()
         self.shutdown_event = threading.Event()
@@ -110,7 +104,7 @@ class GDBServer(threading.Thread):
     def run(self):
         while True:
             new_command = False
-            data = []
+            data = ""
             logging.info('GDB server started')
             
             self.shutdown_event.clear()
@@ -146,7 +140,7 @@ class GDBServer(threading.Thread):
                         if self.shutdown_event.isSet() or self.detach_event.isSet():
                             break
                         self.abstract_socket.setBlocking(0)
-                        data = self.abstract_socket.read()
+                        data += self.abstract_socket.read()
                         if data.index("$") >= 0 and data.index("#") >= 0:
                             break
                     except (ValueError, socket.error):
@@ -248,7 +242,7 @@ class GDBServer(threading.Thread):
         
         else:
             logging.error("Unknown RSP packet: %s", msg)
-            return None
+            return self.createRSPPacket("")
         
     def detach(self, data):
         resp = "OK"
@@ -277,6 +271,16 @@ class GDBServer(threading.Thread):
         self.target.resume()
         self.abstract_socket.setBlocking(0)
         
+        # Try to set break point at hardfault handler to avoid
+        # halting target constantly
+        if (self.target.availableBreakpoint() >= 1):
+            bpSet=True
+            hardfault_handler = self.target.readMemory(4*3)
+            self.target.setBreakpoint(hardfault_handler)
+        else:
+            bpSet=False
+            logging.info("No breakpoint available. Interfere target constantly.")
+        
         val = ''
         
         while True:
@@ -294,17 +298,26 @@ class GDBServer(threading.Thread):
             
             if self.target.getState() == TARGET_HALTED:
                 logging.debug("state halted")
-                val = 'S05'
+                ipsr = self.target.readCoreRegister('xpsr')
+                if (ipsr & 0x1f) == 3:
+                    val = "S" + FAULT[3]
+                else:
+                    val = 'S05'
                 break
             
-            self.target.halt()
-            ipsr = self.target.readCoreRegister('xpsr')
-            logging.debug("GDB resume xpsr: 0x%X", ipsr)
-            if (ipsr & 0x1f) == 3:
-                val = "S" + FAULT[3]
-                break
-            self.target.resume()
+            if not bpSet:
+                # Only do this when no bp available as it slows resume operation
+                self.target.halt()
+                ipsr = self.target.readCoreRegister('xpsr')
+                logging.debug("GDB resume xpsr: 0x%X", ipsr)
+                if (ipsr & 0x1f) == 3:
+                    val = "S" + FAULT[3]
+                    break
+                self.target.resume()
         
+        if bpSet:
+            self.target.removeBreakpoint(hardfault_handler)
+
         self.abstract_socket.setBlocking(1)
         return self.createRSPPacket(val), 0, 0
     
@@ -328,7 +341,8 @@ class GDBServer(threading.Thread):
             return self.createRSPPacket("OK")
         
         elif ops == 'FlashWrite':
-            logging.debug("flash write addr: 0x%s", data.split(':')[1])
+            write_addr = int(data.split(':')[1], 16)
+            logging.debug("flash write addr: 0x%s", write_addr)
             # search for second ':' (beginning of data encoded in the message)
             second_colon = 0
             idx_begin = 0
@@ -336,37 +350,47 @@ class GDBServer(threading.Thread):
                 if data[idx_begin] == ':':
                     second_colon += 1
                 idx_begin += 1
+
+            # if there's gap between sections, fill it
+            flash_watermark = len(self.flashData)
+            pad_size = write_addr - flash_watermark
+            if pad_size > 0:
+                self.flashData += [0xFF] * pad_size
+            
+            # append the new data if it doesn't overlap existing data
+            if write_addr >= flash_watermark:
+                self.flashData += self.unescape(data[idx_begin:len(data) - 3])
+            else:
+                logging.error("Invalid FlashWrite address %d overlaps current data of size %d", write_addr, flash_watermark)
                 
-            self.flashData += data[idx_begin:len(data) - 3]
             return self.createRSPPacket("OK")
         
         # we need to flash everything
         elif 'FlashDone' in ops :
             flashPtr = 0
-            
-            unescaped_data = self.unescape(self.flashData)
-            
-            bytes_to_be_written = len(unescaped_data)
-            
+            bytes_to_be_written = len(self.flashData)
+
             """
             bin = open(os.path.join(parentdir, 'res', 'bad_bin.txt'), "w+")
             
             i = 0
             while (i < bytes_to_be_written):
-                bin.write(str(unescaped_data[i:i+16]) + "\n")
+                bin.write(str(self.flashData[i:i+16]) + "\n")
                 i += 16
             """
-            
-            
+
             logging.info("flashing %d bytes", bytes_to_be_written)
-                    
-            while len(unescaped_data) > 0:
-                size_to_write = min(self.flash.page_size, len(unescaped_data))
-                self.flash.programPage(flashPtr, unescaped_data[:size_to_write])
+
+            while len(self.flashData) > 0:
+                size_to_write = min(self.flash.page_size, len(self.flashData))
+                #if 0 is returned from programPage, security check failed
+                if (self.flash.programPage(flashPtr, self.flashData[:size_to_write]) == 0):
+                    logging.error("Protection bits error, flashing has stopped")
+                    return None
                 flashPtr += size_to_write
-                
-                unescaped_data = unescaped_data[size_to_write:]
-                
+
+                self.flashData = self.flashData[size_to_write:]
+
                 # print progress bar
                 sys.stdout.write('\r')
                 i = int((float(flashPtr)/float(bytes_to_be_written))*20.0)
@@ -376,7 +400,7 @@ class GDBServer(threading.Thread):
                 
             sys.stdout.write("\n\r")
             
-            self.flashData = ""
+            self.flashData = []
             
             """
             bin.close()
@@ -479,10 +503,11 @@ class GDBServer(threading.Thread):
             
     def getRegister(self):
         resp = ''
-        for i in range(len(CORE_REGISTER)):
+        # only core registers are printed
+        for i in sorted(CORE_REGISTER.values())[4:20]:
             reg = self.target.readCoreRegister(i)
             resp += self.intToHexGDB(reg)
-            logging.debug("GDB reg: %s = 0x%X", i, reg)
+            logging.debug("GDB reg: %s = 0x%X", self.target.getRegisterName(i), reg)
         return self.createRSPPacket(resp)
         
     def lastSignal(self):
@@ -514,7 +539,7 @@ class GDBServer(threading.Thread):
             
             elif query[1] == 'memory-map' and query[2] == 'read':
                 data = query[4].split(',')
-                resp = self.handleQueryXML('momery_map', int(data[0], 16), int(data[1].split('#')[0], 16))
+                resp = self.handleQueryXML('memory_map', int(data[0], 16), int(data[1].split('#')[0], 16))
                 return self.createRSPPacket(resp)
                 
             else:
@@ -541,13 +566,13 @@ class GDBServer(threading.Thread):
             return self.createRSPPacket(resp)
         
         else:
-            return None
+            return self.createRSPPacket("")
             
     def handleQueryXML(self, query, offset, size):
         logging.debug('GDB query %s: offset: %s, size: %s', query, offset, size)
         xml = ''
-        if query == 'momery_map':
-            xml = self.flash.memoryMapXML
+        if query == 'memory_map':
+            xml = self.target.memoryMapXML
         elif query == 'read_feature':
             xml = self.target.targetXML
 
@@ -560,7 +585,7 @@ class GDBServer(threading.Thread):
             return
         
         if size > (self.packet_size - 4):
-            size = self.packet_size - 4 
+            size = self.packet_size - 4
         
         nbBytesAvailable = size_xml - offset
         
@@ -582,7 +607,7 @@ class GDBServer(threading.Thread):
             checksum += ord(c)
         checksum = checksum % 256
         checksum = hex(checksum)
-        
+
         if int(checksum[2:], 16) < 0x10:
             resp += '0'
         resp += checksum[2:]
@@ -591,4 +616,4 @@ class GDBServer(threading.Thread):
         return resp
     
     def ack(self):
-        self.abstract_socket.write("+")
\ No newline at end of file
+        self.abstract_socket.write("+")
