@@ -18,12 +18,20 @@
 from cortex_m import CortexM
 
 class LPC1768(CortexM):
+
+    memoryMapXML =  """<?xml version="1.0"?>
+<!DOCTYPE memory-map PUBLIC "+//IDN gnu.org//DTD GDB Memory Map V1.0//EN" "http://sourceware.org/gdb/gdb-memory-map.dtd">
+<memory-map>
+    <memory type="flash" start="0x0" length="0x80000"> <property name="blocksize">0x400</property></memory>
+    <memory type="ram" start="0x10000000" length="0x8000"> </memory>
+    <memory type="ram" start="0x2007C000" length="0x8000"> </memory>
+</memory-map>
+"""
     
     def __init__(self, transport):
-        CortexM.__init__(self, transport)
+        super(LPC1768, self).__init__(transport)
         self.auto_increment_page_size = 0x1000
         
-        
     def reset(self):
         # halt processor
         self.halt()
@@ -37,4 +45,4 @@ class LPC1768(CortexM):
         # not remap 0x0000-0x0020 to anything but the flash
         self.writeMemory(0x400FC040, 1)
         CortexM.resetStopOnReset(self)
-    
\ No newline at end of file
+    
