@@ -15,7 +15,7 @@
  limitations under the License.
 """
 
-class Transport():
+class Transport(object):
     
     def __init__(self, interface):
         self.interface = interface
@@ -61,4 +61,4 @@ class Transport():
         return
     
     def reset(self):
-        return
\ No newline at end of file
+        return
